#version 300 es
precision highp float;

#if __VERSION__ == 300
    #define varying in
    #define texture2D texture
    layout(location = 0) out vec4 FragColor;	
    //precision highp sampler2DArray;
    //precision highp sampler2DShadow;

	#ifdef NormalsRT1
		layout(location = 1) out vec4 NormalRT;
	#endif
	#ifdef DepthRT	
		layout(location = 1) out vec4 DepthOut;
	#endif

#else//ES 2
    #define FragColor gl_FragColor
#endif

#if defined(EnableShadows) && defined(ShadowSamplers) && __VERSION__ == 100 //Not available on iPhone 4(and probably iPad 1)
    #extension GL_EXT_shadow_samplers : require
	#define shadow2D shadow2DEXT
    uniform sampler2DShadow ShadowMap;
#elif __VERSION__ == 100
    uniform sampler2D ShadowMap;
#endif
#if __VERSION__ == 300
    #define shadow2D texture
    uniform highp sampler2DShadow ShadowMap;
#endif


struct Light
{
	vec3 Position;
	int LightType;
	
	vec3 Direction;
	vec3 Attenuation;
	
	vec3 Ambient;
	vec3 Diffuse;
	vec3 Specular;
	float Intensity;
};

#ifdef HasBoneIndices
uniform mat4 BoneMatrix[30];
#endif

#ifdef UseGlobalUniformBuffer
layout(shared) uniform PerFrameBuffer
{
#endif
    uniform mat4 ProjectionMatrix;
    uniform mat4 ViewMatrix;
    
    uniform mat4 WorldMatrix;
    uniform mat4 InverseWorldMatrix;
    uniform mat4 ShadowViewMatrix;
    uniform vec4 MaterialColor;
    
    uniform Light Light0;
#ifdef UseGlobalUniformBuffer
};
#endif

//in GS_OUT
//{
	#ifdef HasTexcoords
		#ifdef IsTextureArraysSupported
			varying vec3 OutTexcoord;
		#else
			varying vec2 OutTexcoord;
		#endif
	#endif

	#ifdef HasNormals
		varying vec3 OutNormal;
	#endif

	varying vec3 WorldPos;
//}Input;

#ifdef IsTextureArraysSupported
	uniform highp sampler2DArray   Texture0;
	uniform float TextureLayer;
#else
	uniform sampler2D   Texture0;
#endif

//#extension GL_EXT_shadow_samplers : require
//#define shadow2D shadow2DEXT

void ComputeLight( Light LightProperties, vec3 WorldPosition, vec3 Normal, vec3 WorldViewVector,
                  inout vec3 TotalAmbient, inout vec3 TotalDiffuse, inout vec3 TotalSpecular)
{
	//Stupid GL camera transform
	vec3 Position = LightProperties.Position;
	//Position.z *=-1;
    
	vec3 LightDirection = Position - WorldPosition;
	float DistanceToLight = length( LightDirection );
	float DistanceToLightSquared = dot(LightDirection, LightDirection);
	vec3 Att = LightProperties.Attenuation;
	float Attenuation = Att.x + Att.y * DistanceToLight + Att.z * DistanceToLightSquared;
	Attenuation = max( 1.0, Attenuation);
	Attenuation = 1.0 / Attenuation;	
    
	LightDirection = normalize( LightDirection );
	float dotres = dot( Normal, LightDirection );
    vec3 NDotL = vec3(dotres,dotres,dotres);
	float Contribution = max( 0.0, dotres );
	
	float SpecularContribution = 0.0;
	/*if (0)
	{
		float SpecularMultiplier = 0.0;
        if (dotres > 0.0)
            SpecularMultiplier = 1.0;
        
		vec3 LightHalfVector = LightDirection + WorldViewVector;
        LightHalfVector = normalize( LightHalfVector );               
        
        float LightNDotH = max(0.0, dot(Normal, LightHalfVector) );
        SpecularContribution = pow( LightNDotH, 64.0 );                
        SpecularContribution = SpecularContribution * SpecularMultiplier * Attenuation;
    }*/
	
	TotalAmbient  += LightProperties.Ambient;
	TotalDiffuse  += Contribution * LightProperties.Diffuse * LightProperties.Intensity;// * Attenuation;
	TotalSpecular += SpecularContribution * LightProperties.Specular;// * Attenuation;
}
float texture2DCompare(sampler2D depths, vec2 uv, float compare)
{
    float depth = texture2D(depths, uv).r;
    return step(compare, depth);
}
float IsShadowed( vec2 Texcoord, float LightDepth, vec2 Offset, vec2 ShadowMapSizeInverse )
{
	vec2 TexOffset = vec2( Offset.x * ShadowMapSizeInverse.x, Offset.y * ShadowMapSizeInverse.y );	
	#ifdef ShadowSamplers
		float Depth = shadow2D( ShadowMap, vec3( Texcoord.x + TexOffset.x,  Texcoord.y + TexOffset.y, LightDepth)  );		
		return 1.0 - Depth;
	#elif defined EnableShadows
		vec2 TexelProj = Texcoord * ShadowMapSize + 0.5 ;
		vec2 f = fract( TexelProj );
		vec2 centroidUV = floor( TexelProj ) / ShadowMapSize;

		float Depth00 = texture2DCompare( ShadowMap, centroidUV + TexOffset, LightDepth);
		float Depth01 = texture2DCompare( ShadowMap, centroidUV + TexOffset + vec2(0.0, ShadowMapSizeInverse.y ) , LightDepth);		
		float Depth10 = texture2DCompare( ShadowMap, centroidUV + TexOffset + vec2(ShadowMapSizeInverse.x, 0.0), LightDepth);
		float Depth11 = texture2DCompare( ShadowMap, centroidUV + TexOffset + ShadowMapSizeInverse, LightDepth );

		float L1 = mix( Depth00, Depth01, f.y);
	    float L2 = mix( Depth10, Depth11, f.y);
	    
		float Depth = mix( L1, L2, f.x);
	
		return 1.0 - Depth;		
	#else
		return 1.0;
	#endif
}
float GetFilteredShadow(vec2 Texcoord, vec2 ShadowMapSizeInverse, float LightSpaceDepth, float GridSize)
{	
	float Shadowed = 0.0;
	float Shadow = 1.0;
	float coef = 1.0;
					
	//float GridSize = ShadowGridSize;//1=3x3,2=5x5
	
	for(float x=-GridSize; x<=GridSize; x++)
		for(float y=-GridSize; y<=GridSize; y++)
		{
			vec2 Offset = vec2( x, y );
			Shadowed += IsShadowed( Texcoord, LightSpaceDepth, Offset, ShadowMapSizeInverse );
		}
	
	coef = (GridSize*2.0+1.0) * (GridSize*2.0+1.0);

	if (Shadowed > 0.0)
	{
		Shadow = 1.0 - Shadowed/coef;
		//Shadow = 1.0 - Shadowed;
	}

	return Shadow;
}
float ComputeShadow(vec2 ShadowTextureSize, float ShadowBias, vec3 WorldPos, mat4 ShadowViewMatrix, float GridSize)
{
	//Optimization, move this to vertex shader !
	vec4 LightProjPos = ShadowViewMatrix * vec4(WorldPos.xyz,1);

	vec3 LightDiv = vec3( LightProjPos.x / LightProjPos.w, LightProjPos.y / LightProjPos.w, LightProjPos.z / LightProjPos.w);
	vec3 ShadowTexC = vec3( 0.5, 0.5, 0.5 );
	ShadowTexC *= LightDiv;
	ShadowTexC += vec3( 0.5, 0.5, 0.5 );

	//Flip Y, only for DX
	//ShadowTexC.y = 1.0 - ShadowTexC.y;

	//float LightSpaceDepth = (LightProjPos.z / LightProjPos.w);//DX Style ?
	float LightSpaceDepth = ShadowTexC.z - ShadowBias;
		
	//float Depth = texture2D( ShadowMap, ShadowTexC.xy ).x;

    if ( LightProjPos.z < 0.0 || LightProjPos.z > 500.0 )
		//|| LightProjPos.x < 0.0 || LightProjPos.x > 1.0
		//|| LightProjPos.y < 0.0 || LightProjPos.y > 1.0)
    {
		return 1.0;//lit
    }
	else
	{
		vec2 ShadowMapSizeInverse = 1.0 / ShadowTextureSize;
		return GetFilteredShadow( ShadowTexC.xy, ShadowMapSizeInverse, LightSpaceDepth, GridSize );
	}
}

void main()
{
	#ifdef EnableCullHeight
	
		if ( WorldPos.y < CullHeight)
			discard;
	
	#endif

    vec3 Normal = vec3(0.0,0.0,1.0);
	vec3 WorldNormal;
    #ifdef HasNormals
		//this is already done in VS
		//WorldNormal = OutNormal * InverseWorldMatrix;
        Normal = normalize( OutNormal );
    #endif
    
    vec3 TotalAmbient = vec3(0.0,0.0,0.0);	
	vec3 TotalDiffuse = vec3(0.0,0.0,0.0);	
	vec3 TotalSpecular = vec3(0.0,0.0,0.0);	

	vec3 SpecularColor = vec3(0.0,0.0,0.0);	
    
    vec3 WorldViewVector = vec3(0.0,-1.0,0.0);//CameraPosition - WorldPos;
	WorldViewVector = normalize( WorldViewVector );
    
    #ifdef HasNormals
        ComputeLight( Light0, WorldPos, Normal, WorldViewVector, TotalAmbient, TotalDiffuse, TotalSpecular );
    #endif
    
    vec3 DiffuseTexture = vec3(0.5, 0.5, 0.5 );
    
    #if defined(HasDiffuseTexture) && defined(HasTexcoords)
		#ifdef IsTextureArraysSupported
			//DiffuseTexture = texture( Texture0, vec3( OutTexcoord, TextureLayer ) ).rgb;
			DiffuseTexture = texture( Texture0, OutTexcoord ).rgb;
		#else
			DiffuseTexture = texture2D( Texture0, OutTexcoord ).rgb;
		#endif
    #endif
    
    float Alpha = MaterialColor.a;
    
    #ifdef EnableAlphaTesting
        float Average = (DiffuseTexture.r + DiffuseTexture.g + DiffuseTexture.b)/3.0;
        if (Average > 0.8)
            discard;
    
        Alpha = mix( 1.0, 0.0, Average );
    #endif
    
     vec3 FinalColor = vec3(0.0, 0.0, 0.0);
    
    #ifdef HasNormals
        FinalColor = TotalAmbient + (TotalDiffuse + SpecularColor* 10.0 * TotalSpecular);
    #else
        FinalColor = vec3(1.0, 1.0, 1.0);
    #endif
	
    #ifdef HasDiffuseTexture		
        FinalColor *= DiffuseTexture;
	#else
        FinalColor *= MaterialColor.rgb;
    #endif

	bool UseShadows = false;
	vec2 ShadowMapSizeCopy;
	#ifdef EnableShadows
		ShadowMapSizeCopy = ShadowMapSize;//textureSize( ShadowMap, 0 );//
		UseShadows = true;
	
	if (UseShadows)
	{		
		float ShadowBias = 0.0;//0.0005;
		float Shadow = ComputeShadow( ShadowMapSizeCopy, ShadowBias, WorldPos, ShadowViewMatrix, ShadowGridSize );

		if (Shadow < 0.5)
			Shadow = 0.5;

		FinalColor *= Shadow;
	}

	#endif
	
	//Debug->
	//FinalColor = TotalAmbient + (TotalDiffuse);// + TotalSpecular);
	//<-

    FragColor = vec4( FinalColor , Alpha );

	#ifdef NormalsRT
		NormalRT = vec4( Normal / 2.0 + 0.5, 1.0 );
	#endif
	#ifdef DepthRT
		DepthOut = gl_FragCoord.z;// / gl_FragCoord.w;
	#endif
	//gl_FragDepth = 0.0;
}

