#version 300 es
#line 2

#if __VERSION__ == 300
    #define attribute in
    #define varying out
#endif

attribute vec4 POSITION;

#ifdef HasTexcoords
	#ifdef IsTextureArraysSupported
		attribute vec3 TEXCOORD;
	#else
		attribute vec2 TEXCOORD;
	#endif
#endif

#ifdef HasNormals
    attribute vec3 NORMAL;
#endif

#ifdef HasBoneIndices
	attribute vec4 BONE_INDICES;
#endif
#ifdef HasBoneWeights
	attribute vec4 BONE_WEIGHTS;
#endif

struct Light
{
	vec3 Position;
	int LightType;
	
	vec3 Direction;
	vec3 Attenuation;
	
	vec3 Ambient;
	vec3 Diffuse;
	vec3 Specular;
	float Intensity;
};

#ifdef HasBoneIndices
uniform mat4 BoneMatrix[30];
#endif

#ifdef UseGlobalUniformBuffer
layout(shared) uniform PerFrameBuffer
{
#endif
    uniform mat4 ProjectionMatrix;
    uniform mat4 ViewMatrix;
    
    uniform mat4 WorldMatrix;
    uniform mat4 InverseWorldMatrix;
    uniform mat4 ShadowViewMatrix;
    uniform vec4 MaterialColor;
    
    uniform Light Light0;
#ifdef UseGlobalUniformBuffer
};
#endif


//out GS_OUT
//{
	#ifdef HasTexcoords
		#ifdef IsTextureArraysSupported
			varying vec3 OutTexcoord;
		#else
			varying vec2 OutTexcoord;
		#endif
	#endif

	#ifdef HasNormals
		varying vec3 OutNormal;
	#endif

	varying vec3 WorldPos;
//}Output;

#ifdef IsSeparateProgramObjectsSupported
	varying vec4 gl_Position;
#endif

void main()
{
	#if defined(HasBoneIndices) && defined(HasBoneWeights)
		mat4 SkinTransform = BoneMatrix[ int(BONE_INDICES.x) ] * BONE_WEIGHTS.x;
	
		SkinTransform += BoneMatrix[ int(BONE_INDICES.y) ] *  BONE_WEIGHTS.y;
		SkinTransform += BoneMatrix[ int(BONE_INDICES.z) ] *  BONE_WEIGHTS.z;
		SkinTransform += BoneMatrix[ int(BONE_INDICES.w) ] *  BONE_WEIGHTS.w;
		
		WorldPos = ( SkinTransform * vec4( POSITION.xyz, 1) ).xyz;
	#else
		WorldPos = POSITION.xyz;
	#endif

	WorldPos =  ( WorldMatrix * vec4( WorldPos, 1) ).xyz;	

    #ifdef HasTexcoords
        OutTexcoord = TEXCOORD;
        OutTexcoord.y = 1.0 - OutTexcoord.y;//ANDROID FIX(for KTX)
	#endif
    
    #ifdef HasNormals
        OutNormal = mat3(InverseWorldMatrix) * NORMAL;
        //Output.OutNormal = normalize( Output.OutNormal );
	#endif
        
    gl_Position = ProjectionMatrix * vec4( WorldPos, 1);
	//gl_Position = vec4(Output.WorldPos, 1);	
}